@@ -34,64 +34,67 @@ bicor = function(x, y = NULL, robustX = TRUE, robustY = TRUE, use = 'all.obs', m
 
   x = as.matrix(x);
   if (prod(dim(x))==0) stop("'x' has a zero dimension."); 
-  nNA = 0;
-  err = 0;
-  warnX = 0;
-  warnY = 0;
+  storage.mode(x) = "double";
+  nNA = 0L;
+  err = 0L;
+  warnX = 0L;
+  warnY = 0L;
+  quick = as.double(quick);
+  maxPOutliers = as.double(maxPOutliers);
+  fallback = as.integer(fallback);
+  cosineX = as.integer(cosineX);
+  robustX = as.integer(robustX);
+  nThreads = as.integer(nThreads);
+  verbose = as.integer(verbose); indent = as.integer(indent)
   if (is.null(y))
   {
     if (!robustX)
     {
-      bi = cor(x, use = use)
+      res = cor(x, use = use)
     } else {
-      bi = matrix(0, ncol(x), ncol(x));
-      res = .C("bicor1Fast", x = as.double(x), nrow = as.integer(nrow(x)), ncol = as.integer(ncol(x)),
-               maxPOutliers = as.double(maxPOutliers), 
-               quick = as.double(quick), 
-               fallback = as.integer(fallback),
-               cosine = as.integer(cosineX), 
-               res = as.double(bi), nNA = as.integer(nNA),
-               err = as.integer(err), 
-               warn = as.integer(warnX), nThreads = as.integer(nThreads),
-               verbose = as.integer(verbose), indent = as.integer(indent),
-               NAOK = TRUE, PACKAGE = "WGCNA");
+      res = .Call("bicor1_call", x, 
+               maxPOutliers, 
+               quick, 
+               fallback,
+               cosineX, 
+               nNA, err, warnX, 
+               nThreads, verbose, indent,
+               PACKAGE = "WGCNA");
     }
-    dim(res$res) = dim(bi);
-    if (!is.null(dimnames(x)[[2]])) dimnames(res$res) = list(dimnames(x)[[2]],  dimnames(x)[[2]] );
-    if (res$warn > 0)
+    if (!is.null(colnames(x))) dimnames(res) = list(colnames(x),  colnames(x));
+    if (warnX > 0)
     {
       # For now have only one warning
       warning(paste("bicor: zero MAD in variable 'x'.", .zeroMADWarnings[fallback]));
     }
   } else {
     y = as.matrix(y);
+    storage.mode(y) = "double";
     if (prod(dim(y))==0) stop("'y' has a zero dimension."); 
     if (nrow(x)!=nrow(y))
       stop("'x' and 'y' have incompatible dimensions (unequal numbers of rows).");
-    bi = matrix(0, ncol(x), ncol(y));
-    res = .C("bicorFast", x = as.double(x), nrow = as.integer(nrow(x)), ncolx = as.integer(ncol(x)),
-             y = as.double(y), ncoly = as.integer(ncol(y)),
-             robustX = as.integer(robustX), robustY = as.integer(robustY),
-             maxPOutliers = as.double(maxPOutliers), 
-             quick = as.double(quick), 
-             fallback = as.integer(fallback),
-             cosineX = as.integer(cosineX),
-             cosineY = as.integer(cosineY),
-             res = as.double(bi), nNA = as.integer(nNA), err = as.integer(err),
-             warnX = as.integer(warnX), 
-             warnY = as.integer(warnY), 
-             nThreads = as.integer(nThreads),
-             verbose = as.integer(verbose), indent = as.integer(indent), NAOK = TRUE,
+    cosineY = as.integer(cosineY);
+    robustY = as.integer(robustY);
+    res = .Call("bicor2_call", x, y,
+             robustX, robustY,
+             maxPOutliers, 
+             quick, 
+             fallback,
+             cosineX,
+             cosineY,
+             nNA, err,
+             warnX, warnY, 
+             nThreads,
+             verbose, indent,
              PACKAGE = "WGCNA");
-    dim(res$res) = dim(bi);
     if (!is.null(dimnames(x)[[2]]) || !is.null(dimnames(y)[[2]]))
-        dimnames(res$res) = list(dimnames(x)[[2]], dimnames(y)[[2]]);
-    if (res$warnX > 0)
+        dimnames(res) = list(dimnames(x)[[2]], dimnames(y)[[2]]);
+    if (warnX > 0)
       warning(paste("bicor: zero MAD in variable 'x'.", .zeroMADWarnings[fallback]));
-    if (res$warnY > 0)
+    if (warnY > 0)
       warning(paste("bicor: zero MAD in variable 'y'.", .zeroMADWarnings[fallback]));
   }
-  if (res$err > 0)
+  if (err > 0)
   {
     if (err > nKnownErrors)
     {
@@ -100,12 +103,12 @@ bicor = function(x, y = NULL, robustX = TRUE, robustY = TRUE, use = 'all.obs', m
       stop(paste(Cerrors[err], "occurred in compiled code. "));
     }
   }
-  if (res$nNA > 0)
+  if (nNA > 0)
   {
     warning(paste("Missing values generated in calculation of bicor.",
                   "Likely cause: too many missing entries, zero median absolute deviation, or zero variance."));
   }
-  res$res;
+  res;
 }
 
 # Code to call my implementation of correlation
@@ -155,9 +158,10 @@ cor = function(x, y = NULL, use = "all.obs", method = c("pearson", "kendall", "s
       if (is.null(nThreads) || (nThreads==0)) nThreads = .useNThreads();
  
       if (prod(dim(x))==0) stop("'x' has a zero dimension."); 
-      nNA = as.integer(0);
-      err = as.integer(0);
-      cosine = as.integer(cosine);
+      storage.mode(x)= "double";
+      nNA = 0L
+      err = 0L
+      cosineX = as.integer(cosineX);
       nThreads = as.integer(nThreads);
       verbose = as.integer(verbose);
       indent = as.integer(indent);
@@ -166,25 +170,23 @@ cor = function(x, y = NULL, use = "all.obs", method = c("pearson", "kendall", "s
          res = .Call("cor1Fast_call", x,
                    quick, cosine,
                    nNA, err, nThreads,
-                   verbose, indent, package = "WGCNA");
+                   verbose, indent, PACKAGE = "WGCNA");
          if (!is.null(dimnames(x)[[2]])) dimnames(res) = list(dimnames(x)[[2]],  dimnames(x)[[2]] );
       } else {
          y = as.matrix(y);
+         storage.mode(y)= "double";
+         cosineY = as.integer(cosineY);
          if (prod(dim(y))==0) stop("'y' has a zero dimension."); 
          if (nrow(x)!=nrow(y))
             stop("'x' and 'y' have incompatible dimensions (unequal numbers of rows).");
-         bi = matrix(0, ncol(x), ncol(y));
-         res = .C("corFast", x = as.double(x), nrow = as.integer(nrow(x)), ncolx = as.integer(ncol(x)),
-                 y = as.double(y), ncoly = as.integer(ncol(y)),
-                 quick = as.double(quick), 
-                 cosineX = as.integer(cosineX), 
-                 cosineY = as.integer(cosineY),
-                 res = as.double(bi), nNA = as.integer(nNA), err = as.integer(err),
-                 nThreads = as.integer(nThreads),
-                 verbose = as.integer(verbose), indent = as.integer(indent), NAOK = TRUE,
+         res = .Call("corFast_call", x, y, 
+                 quick, 
+                 cosineX, 
+                 cosineY,
+                 nNA, err,
+                 nThreads,
+                 verbose, indent, 
                  PACKAGE = "WGCNA");
-         res = res$res;
-         dim(res) = dim(bi);
          if (!is.null(dimnames(x)[[2]]) || !is.null(dimnames(y)[[2]]))
             dimnames(res) = list(dimnames(x)[[2]], dimnames(y)[[2]]);
       }
