@@ -42,34 +42,34 @@ class indArray
 {
   protected:
 
-    int 	* data_;
-    int 	size_;
+    size_t 	* data_;
+    size_t 	size_;
     int 	allocated;
     string	name_;
 
-    int	val32[2]; 
-    int	mask[8*sizeof(int)];
-    int	invMask[8*sizeof(int)];
+    size_t	val32[2]; 
+    size_t	mask[8*sizeof(size_t)];
+    size_t	invMask[8*sizeof(size_t)];
 
   public:
 
 #ifdef CheckDimensions
 
-    bool value(int i)
+    bool value(size_t i)
       {
-        int ii = (i/(8*sizeof(int)));
+        size_t ii = (i/(8*sizeof(size_t)));
         if (ii >= size_)
           throw(Exception(string("indArray::value: index out of range in variable") + name()));
-        int j = (i % (8*sizeof(int)));
+        size_t j = (i % (8*sizeof(size_t)));
         return ((data_[ii] & mask[j]) != 0);
       }
 
-    void value(int i, bool v)
+    void value(size_t i, bool v)
       {
-        int ii = (i/(8*sizeof(int)));
+        size_t ii = (i/(8*sizeof(size_t)));
         if (ii >= size_)
           throw(Exception(string("indArray::value: index out of range in variable") + name()));
-        int j = (i % (8*sizeof(int)));
+        size_t j = (i % (8*sizeof(size_t)));
         //cout << "value: i: " << i << ", ii: " << ii << ", j: " << j << ", v:" << v << endl;
         if (v)
           data_[ii] |= mask[j];
@@ -79,17 +79,17 @@ class indArray
 
 #else
 
-    bool value(int i)
+    bool value(size_t i)
       {
-        int ii = (i/(8*sizeof(int)));
-        int j = (i % (8*sizeof(int)));
+        size_t ii = (i/(8*sizeof(size_t)));
+        size_t j = (i % (8*sizeof(size_t)));
         return (data_[ii] & mask[j] != 0);
       }
 
-    void value(int i, bool v)
+    void value(size_t i, bool v)
       {
-        int ii = (i/(8*sizeof(int)));
-        int j = (i % (8*sizeof(int)));
+        size_t ii = (i/(8*sizeof(size_t)));
+        size_t j = (i % (8*sizeof(size_t)));
         if (v)
           data_[ii] |= mask[j];
         else
@@ -102,32 +102,32 @@ class indArray
     void name(string n) {name_ = n; }
     string name() {return name_; }
 
-    void init(int size);
-    void init(int size, bool value);
-    int size() { return size_ * 8 * sizeof(int); }
-    int * data() { return data_; }
+    void init(size_t size);
+    void init(size_t size, bool value);
+    size_t size() { return size_ * 8 * sizeof(size_t); }
+    size_t * data() { return data_; }
 
     void show()
      {
        cout << "data_:";
-       for (int i=0; i<size_; i++)
+       for (size_t i=0; i<size_; i++)
          cout << data_[i] << ", ";
        cout << endl;
        //cout << "mask";
-       //for (int i=0; i<(8*sizeof(int)); i++)
+       //for (size_t i=0; i<(8*sizeof(size_t)); i++)
          //cout << mask[i] << ", ";
        //cout << endl;
        //cout << "invMask";
-       //for (int i=0; i<(8*sizeof(int)); i++)
+       //for (size_t i=0; i<(8*sizeof(size_t)); i++)
          //cout << invMask[i] << ", ";
        //cout << endl;
      }
 
-    indArray() { allocated = 0; data_ = (int *) NULL; }
+    indArray() { allocated = 0; data_ = (size_t *) NULL; }
 
-    indArray(int size) { init(size); }
+    indArray(size_t size) { init(size); }
 
-    indArray(int size, bool value) { init(size, value); }
+    indArray(size_t size, bool value) { init(size, value); }
 
     // indArray(indArray arr);      // This constructor will copy the data from arr into *this
 
@@ -135,28 +135,28 @@ class indArray
 
 };
 
-void indArray::init(int size)
+void indArray::init(size_t size)
 {
-  unsigned val = 1;
-  for (unsigned i=0; i<8*sizeof(int); i++)
+  size_t val = 1;
+  for (size_t i=0; i<8*sizeof(size_t); i++)
   {
     mask[i] = val; 
     invMask[i] = ~mask[i];
-    if (i < 8*sizeof(int)-1) val*=2;
+    if (i < 8*sizeof(size_t)-1) val*=2;
   }
   val32[0] = 0;
   val32[1] = ~0;
-  size_ = (int)(ceil((size*1.0)/(8*sizeof(int))));
-  data_ = new int[size_];
+  size_ = (size_t)(ceil((size*1.0)/(8*sizeof(size_t))));
+  data_ = new size_t[size_];
   allocated = 1;
 }
 
 
-void indArray::init(int size, bool value)
+void indArray::init(size_t size, bool value)
 {
   init(size);
-  for (int i=0; i<size_; i++)
-    data_[i] = val32[(int) value];
+  for (size_t i=0; i<size_; i++)
+    data_[i] = val32[(size_t) value];
 }
 
 #define INT_CLASS iArray
@@ -180,7 +180,7 @@ double max(vector <double> v)
   if (v.size()==0)
     throw(Exception(string("attempt to calculate max of an empty vector.")));
   double max = v[0];
-  for (unsigned i=1; i<v.size(); i++)
+  for (size_t i=1; i<v.size(); i++)
     if (!ISNAN(v[i]) && (v[i] > max)) max = v[i];
   return max;
 }
@@ -190,7 +190,7 @@ int max(vector <int> v)
   if (v.size()==0)
     throw(Exception(string("attempt to calculate max of an empty vector.")));
   int max = v[0];
-  for (unsigned i=1; i<v.size(); i++)
+  for (size_t i=1; i<v.size(); i++)
     if (v[i] > max) max = v[i];
   return max;
 }
@@ -201,7 +201,7 @@ double min(vector <double> v)
   if (v.size()==0)
     throw(Exception(string("attempt to calculate min of an empty vector.")));
   double min = v[0];
-  for (unsigned i=1; i<v.size(); i++)
+  for (size_t i=1; i<v.size(); i++)
     if (!ISNAN(v[i]) && (v[i] < min)) min = v[i];
   return min;
 }
@@ -211,7 +211,7 @@ int min(vector <int> v)
   if (v.size()==0)
     throw(Exception(string("attempt to calculate min of an empty vector.")));
   int min = v[0];
-  for (unsigned i=1; i<v.size(); i++)
+  for (size_t i=1; i<v.size(); i++)
     if (v[i] < min) min = v[i];
   return min;
 }
@@ -226,7 +226,7 @@ void dArray::colQuantile(double q, dArray & quant)
   else
     quant.setDim(dim(), 1);
 
-  int colLen = dim()[0], totLen = length();
+  size_t colLen = dim()[0], totLen = length();
 
   if (colLen==0)
     throw(Exception(string("colQuantile: Column length is zero in variable") + name()));
@@ -236,7 +236,7 @@ void dArray::colQuantile(double q, dArray & quant)
 
   int err;
   double val;
-  for (int i=0, col=0; i<totLen; i+=colLen, col++)
+  for (size_t i=0, col=0; i<totLen; i+=colLen, col++)
   {
     // column.clear();
     copy2vector(i, colLen, column);
@@ -245,6 +245,43 @@ void dArray::colQuantile(double q, dArray & quant)
   }
 }
 
+void dArray::rowQuantile(double q, dArray & quant)
+{
+  if (dim().size()==0)
+    throw(Exception(string(
+       "Attempt to calculate row-wise quantile of array that has no dimensions set.")));
+  if (dim().size()==1)
+    quant.setDim(1);
+  else {
+    if (dim().size()>2)
+      throw(Exception(string(
+       "Row-wise quantiles are only defined for 2-dimensional arrays.")));
+    vector <size_t> dim1 = dim();
+    dim1.pop_back();
+    quant.setDim(dim1);
+  }
+
+  size_t rowLen = dim()[1], nrow = dim()[0];
+
+  if (rowLen==0)
+    throw(Exception(string("rowQuantile: Row length is zero in variable") + name()));
+
+  vector <double> rowData;
+  rowData.reserve(rowLen);
+
+  int err;
+  double val;
+  for (size_t row=0; row<nrow; row++)
+  {
+    rowData.clear();
+    for (size_t col=0; col<rowLen; col++) rowData.push_back(value(row, col));
+    val = quantile(&(rowData[0]), rowLen, q, 0,  & err);
+    quant.linValue(row, val);
+  }
+}
+
+
+
 void iArray::colQuantile(double q, dArray & quant)
 {
   if (dim().size()==0)
@@ -255,7 +292,7 @@ void iArray::colQuantile(double q, dArray & quant)
   else
     quant.setDim(dim(), 1);
 
-  int colLen = dim()[0], totLen = length();
+  size_t colLen = dim()[0], totLen = length();
 
   if (colLen==0)
     throw(Exception(string("colQuantile: Column length is zero in variable") + name()));
@@ -265,7 +302,7 @@ void iArray::colQuantile(double q, dArray & quant)
 
   int err;
   double val;
-  for (int i=0, col=0; i<totLen; i+=colLen, col++)
+  for (size_t i=0, col=0; i<totLen; i+=colLen, col++)
   {
     // column.clear();
     copy2vector(i, colLen, column);
